@@ -1,7 +1,19 @@
-/* Written 2003,2004 by Andi Kleen, SuSE Labs.
- * 
- * Command line NUMA policy control.
- */
+/* Copyright (C) 2003,2004,2005 Andi Kleen, SuSE Labs.
+   Command line NUMA policy control.
+
+   numactl is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public
+   License as published by the Free Software Foundation; version
+   2.
+
+   numactl is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should find a copy of v2 of the GNU General Public License somewhere
+   on your Linux system; if not, write to the Free Software Foundation, 
+   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
 #define _GNU_SOURCE
 #include <getopt.h>
 #include <errno.h>
@@ -22,6 +34,8 @@ struct option opts[] = {
 	{"interleave", 1, 0, 'i' },
 	{"preferred", 1, 0, 'p' },
 	{"cpubind", 1, 0, 'c' }, 
+	{"cpunodebind", 1, 0, 'N' }, 
+	{"physcpubind", 1, 0, 'C' }, 
 	{"membind", 1, 0, 'm'},
 	{"show", 0, 0, 's' }, 
 	{"localalloc", 0,0, 'l'},
@@ -44,7 +58,8 @@ struct option opts[] = {
 void usage(void)
 {
 	fprintf(stderr,
-		"usage: numactl [--interleave=nodes] [--preferred=node] [--cpubind=nodes]\n" 
+		"usage: numactl [--interleave=nodes] [--preferred=node]\n"
+		"               [--physcpubind=cpus] [--cpunodebind=nodes]\n"
 		"               [--membind=nodes] [--localalloc] command args ...\n"
 		"       numactl [--show]\n"
 		"       numactl [--hardware]\n"
@@ -54,7 +69,11 @@ void usage(void)
 		"               memory policy\n"
 		"\n"
 		"memory policy is --interleave, --preferred, --membind, --localalloc\n"
-		"nodes is a comma delimited list of node numbers or none/all.\n"
+		"nodes is a comma delimited list of node numbers or A-B ranges or none/all.\n"
+		"cpus is a comma delimited list of cpu numbers or A-B ranges or all\n"
+		"all ranges can be inverted with !\n"
+		"the old --cpubind argument is deprecated.\n"
+		"use --cpunodebind or --physcpubind instead\n"
 		"length can have g (GB), m (MB) or k (KB) suffixes\n");
 	exit(1);
 } 
@@ -69,22 +88,42 @@ void usage_msg(char *msg, ...)
 	usage();
 } 
 
+void show_physcpubind(void)
+{
+	int ncpus = 8192;
+	
+	for (;;) { 
+		int cpubufsize = round_up(ncpus, BITS_PER_LONG) / BYTES_PER_LONG;
+		unsigned  long cpubuf[cpubufsize / sizeof(long)];
+		
+		memset(cpubuf,0,cpubufsize);
+		if (numa_sched_getaffinity(0, cpubufsize, cpubuf) < 0) { 
+			if (errno == EINVAL && ncpus < 1024*1024) {
+				ncpus *= 2; 
+				continue;
+			}
+			err("sched_get_affinity");
+		}
+		printcpumask("physcpubind", cpubuf, cpubufsize);
+		break;
+	}
+}
+
 void show(void)
 {
 	unsigned long prefnode;
 	nodemask_t membind, interleave, cpubind;
 	unsigned long cur;
 	int policy;
-
-	nodemask_zero(&cpubind);
-	if (numa_sched_getaffinity(getpid(), sizeof(cpubind), 
-				   (unsigned long *)&cpubind) < 0) 
-		err("get_affinity");
+	
 	if (numa_available() < 0) { 
+		show_physcpubind();
 		printf("No NUMA support available on this system.\n");
 		exit(1);
 	}
 
+	cpubind = numa_get_run_node_mask();
+
 	prefnode = numa_preferred();
 	interleave = numa_get_interleave_mask();
 	membind = numa_get_membind();
@@ -96,22 +135,64 @@ void show(void)
 	 
 	printf("policy: %s\n", policy_name(policy));
 		
-	printf("preferred node: %ld\n", prefnode);
-	printmask("interleavemask", &interleave);
-	printf("interleavenode: %ld\n", cur); 
+	printf("preferred node: ");
+	switch (policy) { 
+	case MPOL_PREFERRED:
+		if (prefnode != -1) { 
+			printf("%ld\n", prefnode);
+			break;
+		}
+		/*FALL THROUGH*/
+	case MPOL_DEFAULT:
+		printf("current\n");
+		break;
+	case MPOL_INTERLEAVE:
+		printf("%ld (interleave next)\n",cur); 
+		break; 
+	case MPOL_BIND:
+		printf("%d\n", find_first_bit(&membind, NUMA_NUM_NODES)); 
+		break;
+	} 
+	if (policy == MPOL_INTERLEAVE) {
+		printmask("interleavemask", &interleave);
+		printf("interleavenode: %ld\n", cur); 
+	}
+	show_physcpubind();
+	printmask("cpubind", &cpubind);  // for compatibility
 	printmask("nodebind", &cpubind);
 	printmask("membind", &membind);
 }
 
-char *fmt_mem(unsigned long mem, char *buf) 
+char *fmt_mem(unsigned long long mem, char *buf) 
 { 
 	if (mem == -1L)
 		sprintf(buf, "<not available>"); 
 	else
-		sprintf(buf, "%lu MB", mem >> 20); 
+		sprintf(buf, "%Lu MB", mem >> 20); 
 	return buf;
 } 
 
+static void print_distances(int maxnode)
+{
+	int i,k;
+
+	if (numa_distance(maxnode,0) == 0) {
+		printf("No distance information available.\n");
+		return;
+	}
+	printf("node distances:\n"); 
+	printf("node ");
+	for (i = 0; i <= maxnode; i++) 
+		printf("% 3d ", i);
+	printf("\n");
+	for (i = 0; i <= maxnode; i++) {
+		printf("% 3d: ", i);
+		for (k = 0; k <= maxnode; k++) 
+			printf("% 3d ", numa_distance(i,k)); 
+		printf("\n");
+	}			
+}
+
 void hardware(void)
 { 
 	int i;
@@ -119,11 +200,12 @@ void hardware(void)
 	printf("available: %d nodes (0-%d)\n", 1+maxnode, maxnode); 	
 	for (i = 0; i <= maxnode; i++) { 
 		char buf[64];
-		unsigned long fr;
-		unsigned long sz = numa_node_size(i, &fr); 
+		long long fr;
+		unsigned long long sz = numa_node_size64(i, &fr); 
 		printf("node %d size: %s\n", i, fmt_mem(sz, buf));
 		printf("node %d free: %s\n", i, fmt_mem(fr, buf));
 	}
+	print_distances(maxnode);
 } 
 
 void checkerror(char *s)
@@ -211,6 +293,7 @@ int main(int ac, char **av)
 				numa_set_interleave_mask(&mask);
 			checkerror("setting interleave mask");
 			break;
+		case 'N': /* --cpunodebind */
 		case 'c': /* --cpubind */
 			mask = nodemask(optarg);
 			errno = 0;
@@ -219,6 +302,19 @@ int main(int ac, char **av)
 			numa_run_on_node_mask(&mask);
 			checkerror("sched_setaffinity");
 			break;
+		case 'C': /* --physcpubind */
+		{
+			int ncpus;
+			unsigned long *cpubuf;
+			cpubuf = cpumask(optarg, &ncpus);
+			errno = 0;
+			check_cpubind(do_shm);
+			did_cpubind = 1;
+			numa_sched_setaffinity(0, CPU_BYTES(ncpus), cpubuf);
+			checkerror("sched_setaffinity");
+			free(cpubuf);
+			break;
+		}
 		case 'm': /* --membind */
 			checknuma();
 			setpolicy(MPOL_BIND);
@@ -236,7 +332,9 @@ int main(int ac, char **av)
 		case 'p': /* --preferred */
 			checknuma();
 			setpolicy(MPOL_PREFERRED);
-			arg = strtoul(optarg,NULL,0); 
+			arg = strtoul(optarg,&end,0); 
+			if (*end || end == optarg) 
+				usage();
 			errno = 0;
 			numa_set_bind_policy(0);
 			nodemask_zero(&mask);
@@ -277,7 +375,7 @@ int main(int ac, char **av)
 		case 'M': /* --shmmode */
 			noshm("--shmmode");
 			shmmode = strtoul(optarg, &end, 8);
-			if (end == optarg) 
+			if (end == optarg || *end) 
 				usage();
 			break;
 		case 'd': /* --dump */
@@ -291,9 +389,8 @@ int main(int ac, char **av)
 			numa_set_strict(1);
 			break;
 		case 'I': /* --shmid */
-			noshm("--shmid");
 			shmid = strtoul(optarg, &end, 0);
-			if (end == optarg) 
+			if (end == optarg || *end) 
 				usage();
 			break;
 
