@@ -1,15 +1,19 @@
 #ifndef NUMAIF_H
 #define NUMAIF_H 1
 
+#ifdef __cplusplus
+extern "C" { 
+#endif
+
 /* Kernel interface for NUMA API */
 
 /* System calls */
 extern long get_mempolicy(int *policy, 
-			  unsigned long *nmask, unsigned long maxnode,
+			  const unsigned long *nmask, unsigned long maxnode,
 			  void *addr, int flags);
 extern long mbind(void *start, unsigned long len, int mode, 
-		  unsigned long *nmask, unsigned long maxnode, unsigned flags);
-extern long set_mempolicy(int mode, unsigned long *nmask, 
+		  const unsigned long *nmask, unsigned long maxnode, unsigned flags);
+extern long set_mempolicy(int mode, const unsigned long *nmask, 
 			  unsigned long maxnode);
 
 /* Policies */
@@ -27,4 +31,8 @@ extern long set_mempolicy(int mode, unsigned long *nmask,
 /* Flags for mbind */
 #define MPOL_MF_STRICT  (1<<0)  /* Verify existing pages in the mapping */
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
