@@ -1,7 +1,7 @@
 
 extern int shmfd;
 extern long shmid;
-extern void *shmptr;
+extern char *shmptr;
 extern unsigned long long shmlen;
 extern mode_t shmmode;
 extern unsigned long long shmoffset;
@@ -11,3 +11,6 @@ extern void dump_shm(void);
 extern void attach_shared(char *name);
 extern void attach_sysvshm(char *name); 
 extern void verify_shm(int policy, nodemask_t nodes);
+
+/* in numactl.c */
+extern int exitcode;
